import React, { useCallback, createElement, useMemo, useEffect } from 'react';
import { DefaultElement, DefaultLeaf, Editable, Slate, useSlateStatic, useSlate, withReact } from 'slate-react';
import create from 'zustand';
import { createEditor, Editor, Operation, Path, Element, Text } from 'slate';

function unwrapExports (x) {
	return x && x.__esModule && Object.prototype.hasOwnProperty.call(x, 'default') ? x['default'] : x;
}

function createCommonjsModule(fn, module) {
	return module = { exports: {} }, fn(module, module.exports), module.exports;
}

var vanilla = createCommonjsModule(function (module, exports) {

Object.defineProperty(exports, '__esModule', { value: true });

function create(createState) {
  var state;
  var listeners = new Set();

  var setState = function setState(partial, replace) {
    var nextState = typeof partial === 'function' ? partial(state) : partial;

    if (nextState !== state) {
      var previousState = state;
      state = replace ? nextState : Object.assign({}, state, nextState);
      listeners.forEach(function (listener) {
        return listener(state, previousState);
      });
    }
  };

  var getState = function getState() {
    return state;
  };

  var subscribeWithSelector = function subscribeWithSelector(listener, selector, equalityFn) {
    if (selector === void 0) {
      selector = getState;
    }

    if (equalityFn === void 0) {
      equalityFn = Object.is;
    }

    var currentSlice = selector(state);

    function listenerToAdd() {
      var nextSlice = selector(state);

      if (!equalityFn(currentSlice, nextSlice)) {
        var previousSlice = currentSlice;
        listener(currentSlice = nextSlice, previousSlice);
      }
    }

    listeners.add(listenerToAdd); // Unsubscribe

    return function () {
      return listeners.delete(listenerToAdd);
    };
  };

  var subscribe = function subscribe(listener, selector, equalityFn) {
    if (selector || equalityFn) {
      return subscribeWithSelector(listener, selector, equalityFn);
    }

    listeners.add(listener); // Unsubscribe

    return function () {
      return listeners.delete(listener);
    };
  };

  var destroy = function destroy() {
    return listeners.clear();
  };

  var api = {
    setState: setState,
    getState: getState,
    subscribe: subscribe,
    destroy: destroy
  };
  state = createState(setState, getState, api);
  return api;
}

exports.default = create;
});

var createVanillaStore = unwrapExports(vanilla);

/**
 * Slate plugins vanilla store.
 * @see zustand vanilla store
 */

const slatePluginsStore = createVanillaStore(() => ({}));
/**
 * Slate plugins store.
 * @see zustand store
 */

const useSlatePluginsStore = create(slatePluginsStore);

/**
 * Slate editor with generic type (default is `ReactEditor & HistoryEditor & RandomKeyEditor`).
 */

const useSPEditor = (id = 'main') => useSlatePluginsStore(useCallback(state => {
  var _a;

  return (_a = state[id]) === null || _a === void 0 ? void 0 : _a.editor;
}, [id]));
const useSlatePluginsValue = (id = 'main') => useSlatePluginsStore(useCallback(state => {
  var _a, _b;

  return (_b = (_a = state[id]) === null || _a === void 0 ? void 0 : _a.value) !== null && _b !== void 0 ? _b : [];
}, [id]));
const usePlugins = (id = 'main') => useSlatePluginsStore(useCallback(state => {
  var _a, _b;

  return (_b = (_a = state[id]) === null || _a === void 0 ? void 0 : _a.plugins) !== null && _b !== void 0 ? _b : [];
}, [id]));
const useSlatePluginsPluginKeys = (id = 'main') => useSlatePluginsStore(useCallback(state => {
  var _a, _b;

  return (_b = (_a = state[id]) === null || _a === void 0 ? void 0 : _a.pluginKeys) !== null && _b !== void 0 ? _b : [];
}, [id]));
const useSlatePluginsOptions = (id = 'main') => useSPEditor(id).options;

const flatMapKey = (arr, key) => arr.flatMap(item => {
  var _a;

  return (_a = item[key]) !== null && _a !== void 0 ? _a : [];
});

/**
 * @see {@link Decorate}
 */

const decoratePlugins = (editor, plugins) => entry => {
  let ranges = [];

  const addRanges = newRanges => {
    if (newRanges === null || newRanges === void 0 ? void 0 : newRanges.length) ranges = [...ranges, ...newRanges];
  };

  flatMapKey(plugins, 'decorate').forEach(decorate => {
    addRanges(decorate(editor)(entry));
  });
  return ranges;
};

/**
 * @see {@link OnDOMBeforeInput}
 */

const onDOMBeforeInputPlugins = (editor, plugins) => event => {
  flatMapKey(plugins, 'onDOMBeforeInput').some(onDOMBeforeInput => onDOMBeforeInput(editor)(event) === false);
};

/**
 * @see {@link OnKeyDown}
 */

const onKeyDownPlugins = (editor, plugins) => event => {
  flatMapKey(plugins, 'onKeyDown').some(onKeyDown => onKeyDown(editor)(event) === false);
};

/**
 * @see {@link RenderElement}
 */

const renderElementPlugins = (editor, plugins) => elementProps => {
  let element;
  flatMapKey(plugins, 'renderElement').some(renderElement => {
    element = renderElement(editor)(elementProps);
    return !!element;
  });
  if (element) return element;
  return /*#__PURE__*/createElement(DefaultElement, Object.assign({}, elementProps));
};

/**
 * @see {@link RenderLeaf}
 */

const renderLeafPlugins = (editor, plugins) => props => {
  const leafProps = Object.assign({}, props); // workaround for children readonly error.

  flatMapKey(plugins, 'renderLeaf').forEach(renderLeaf => {
    if (!renderLeaf) return;
    const newChildren = renderLeaf(editor)(leafProps);

    if (newChildren !== undefined) {
      leafProps.children = newChildren;
    }
  });
  return /*#__PURE__*/createElement(DefaultLeaf, Object.assign({}, leafProps));
};

const useEditableProps = ({
  id,
  editableProps
}) => {
  const editor = useSPEditor(id);
  const plugins = usePlugins(id);
  const props = useMemo(() => ({
    renderElement: renderElementPlugins(editor, plugins),
    renderLeaf: renderLeafPlugins(editor, plugins),
    onKeyDown: onKeyDownPlugins(editor, plugins),
    decorate: decoratePlugins(editor, plugins),
    onDOMBeforeInput: onDOMBeforeInputPlugins(editor, plugins)
  }), [editor, plugins]);
  return useCallback(() => Object.assign(Object.assign({}, props), editableProps), [editableProps, props]);
};

const getSlatePluginWithOverrides = withOverrides => options => ({
  withOverrides: withOverrides(options)
});

function pipe(x, ...fns) {
  return fns.reduce((y, fn) => fn(y), x);
}

/**
 * Checks if `value` is classified as an `Array` object.
 *
 * @static
 * @memberOf _
 * @since 0.1.0
 * @category Lang
 * @param {*} value The value to check.
 * @returns {boolean} Returns `true` if `value` is an array, else `false`.
 * @example
 *
 * _.isArray([1, 2, 3]);
 * // => true
 *
 * _.isArray(document.body.children);
 * // => false
 *
 * _.isArray('abc');
 * // => false
 *
 * _.isArray(_.noop);
 * // => false
 */
var isArray = Array.isArray;

var isArray_1 = isArray;

/**
 * Casts `value` as an array if it's not one.
 *
 * @static
 * @memberOf _
 * @since 4.4.0
 * @category Lang
 * @param {*} value The value to inspect.
 * @returns {Array} Returns the cast array.
 * @example
 *
 * _.castArray(1);
 * // => [1]
 *
 * _.castArray({ 'a': 1 });
 * // => [{ 'a': 1 }]
 *
 * _.castArray('abc');
 * // => ['abc']
 *
 * _.castArray(null);
 * // => [null]
 *
 * _.castArray(undefined);
 * // => [undefined]
 *
 * _.castArray();
 * // => []
 *
 * var array = [1, 2, 3];
 * console.log(_.castArray(array) === array);
 * // => true
 */
function castArray() {
  if (!arguments.length) {
    return [];
  }
  var value = arguments[0];
  return isArray_1(value) ? value : [value];
}

var castArray_1 = castArray;

/**
 * Merge and register all the inline types and void types from the plugins and options,
 * using `editor.isInline` and `editor.isVoid`
 */

const withInlineVoid = ({
  plugins = [],
  inlineTypes = [],
  voidTypes = []
}) => editor => {
  const {
    isInline
  } = editor;
  const {
    isVoid
  } = editor;
  let allInlineTypes = [...inlineTypes];
  let allVoidTypes = [...voidTypes];
  plugins.forEach(plugin => {
    if (plugin.inlineTypes) {
      allInlineTypes = allInlineTypes.concat(castArray_1(plugin.inlineTypes(editor)));
    }

    if (plugin.voidTypes) {
      allVoidTypes = allVoidTypes.concat(castArray_1(plugin.voidTypes(editor)));
    }
  });

  editor.isInline = element => {
    return allInlineTypes.includes(element.type) ? true : isInline(element);
  };

  editor.isVoid = element => allVoidTypes.includes(element.type) ? true : isVoid(element);

  return editor;
};
/**
 * @see {@link withInlineVoid}
 */

const useInlineVoidPlugin = getSlatePluginWithOverrides(withInlineVoid);

/**
 * Apply `withInlineVoid` and all slate plugins `withOverrides`.
 * Overrides:
 * - `id`: id of the editor.
 * - `key`: random key for the <Slate> component so each time the editor is created, the component resets.
 * - `options`: {@link SlatePluginsOptions}
 */

const withSlatePlugins = ({
  id = 'main',
  plugins = [],
  options = {},
  components = {}
} = {}) => e => {
  var _a;

  let editor = e;
  editor.id = id;

  if (components) {
    // Merge components into options
    Object.keys(components).forEach(key => {
      options[key] = Object.assign({
        component: components[key]
      }, options[key]);
    });
  } // Default option type is the plugin key


  Object.keys(options).forEach(key => {
    if (options[key].type === undefined) options[key].type = key;
  });
  editor.options = options;

  if (!editor.key) {
    editor.key = Math.random();
  } // Plugins inline and void types


  editor = pipe(editor, withInlineVoid({
    plugins
  })); // Plugins withOverrides

  const withOverrides = (_a = plugins === null || plugins === void 0 ? void 0 : plugins.flatMap(p => {
    var _a;

    return (_a = p.withOverrides) !== null && _a !== void 0 ? _a : [];
  })) !== null && _a !== void 0 ? _a : [];
  editor = pipe(editor, ...withOverrides);
  return editor;
};
/**
 * @see {@link withSlatePlugins}
 */

const useSlatePluginsPlugin = getSlatePluginWithOverrides(withSlatePlugins);

const getInitialState = () => ({
  plugins: [],
  value: [{
    children: [{
      text: ''
    }]
  }],
  pluginKeys: []
});

const set$1 = slatePluginsStore.setState;
const getStateById = (state, id) => {
  var _a;

  return (_a = state[id]) !== null && _a !== void 0 ? _a : getInitialState();
};
const mergeState = (stateToMerge, stateId) => {
  set$1(state => ({
    [stateId]: Object.assign(Object.assign({}, getStateById(state, stateId)), stateToMerge)
  }));
};
const getSetStateByKey = (key, stateId) => (value, id = stateId) => set$1(state => ({
  [id]: Object.assign(Object.assign({}, getStateById(state, id)), {
    [key]: value
  })
}));

const s = slatePluginsStore;
const get = s.getState;
const set = s.setState;
const useSlatePluginsActions = (storeId = 'main') => useMemo(() => {
  const setEditor = getSetStateByKey('editor', storeId);
  return {
    setEditor,
    clearState: (id = storeId) => {
      return set(state => {
        delete state[id];
        return state;
      });
    },
    setInitialState: (id = storeId) => set(state => state[id] ? state : {
      [id]: getInitialState()
    }),
    resetEditorKey: (id = storeId) => {
      const {
        editor
      } = get()[id];
      const {
        plugins
      } = get()[id];
      setEditor(pipe(createEditor(), withSlatePlugins({
        id,
        plugins,
        options: editor === null || editor === void 0 ? void 0 : editor.options
      })), id);
    },
    setPlugins: getSetStateByKey('plugins', storeId),
    setValue: getSetStateByKey('value', storeId),
    setPluginKeys: getSetStateByKey('pluginKeys', storeId)
  };
}, [storeId]);

/**
 * Effects to update the slate plugins store from the options.
 * Dynamically updating the options will update the store state.
 */

const useSlatePluginsEffects = ({
  id,
  value,
  editor,
  components,
  options,
  initialValue,
  plugins
}) => {
  const {
    setInitialState,
    setValue,
    setEditor,
    setPlugins,
    setPluginKeys,
    clearState
  } = useSlatePluginsActions(id);
  const storeEditor = useSPEditor(id);
  useEffect(() => {
    setInitialState();
  }, [setInitialState]); // Clear the state by id on unmount.

  useEffect(() => () => {
    clearState();
  }, [clearState]); // Slate.value

  useEffect(() => {
    initialValue && setValue(initialValue); // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [setValue]); // Slate.value

  useEffect(() => {
    value && setValue(value);
  }, [setValue, value]); // Slate.editor

  useEffect(() => {
    if (storeEditor) return;
    setEditor(pipe(createEditor(), withSlatePlugins({
      id,
      plugins,
      options,
      components
    })));
  }, [storeEditor, components, editor, id, options, plugins, setEditor]); // Slate plugins

  useEffect(() => {
    plugins && setPlugins(plugins);
  }, [plugins, setPlugins]);
  useEffect(() => {
    plugins && setPluginKeys(flatMapKey(plugins, 'pluginKeys'));
  }, [plugins, setPluginKeys]);
};

const onChangePlugins = (editor, plugins) => nodes => {
  flatMapKey(plugins, 'onChange').some(onChange => onChange(editor)(nodes) === false);
};

/**
 * Get Slate props stored in a global store.
 */

const useSlateProps = ({
  id,
  onChange: _onChange
} = {}) => {
  const {
    setValue
  } = useSlatePluginsActions(id);
  const editor = useSPEditor(id);
  const value = useSlatePluginsValue(id);
  const plugins = usePlugins(id);
  const onChange = useMemo(() => newValue => {
    setValue(newValue);
    onChangePlugins(editor, plugins)(newValue);
    _onChange === null || _onChange === void 0 ? void 0 : _onChange(newValue);
  }, [_onChange, editor, plugins, setValue]);
  return useCallback(() => ({
    key: editor === null || editor === void 0 ? void 0 : editor.key,
    editor,
    onChange,
    value
  }), [editor, onChange, value]);
};

/**
 * Run `useSlatePluginsEffects` and props getter for `Slate` and `Editable` components.
 * Use `useSlatePluginsStore` to select store state.
 */

const useSlatePlugins = ({
  id,
  components,
  editor,
  initialValue,
  value,
  options,
  plugins,
  onChange,
  editableProps
}) => {
  useSlatePluginsEffects({
    id,
    components,
    plugins,
    initialValue,
    editor,
    value,
    options
  });
  return {
    getSlateProps: useSlateProps({
      id,
      onChange
    }),
    getEditableProps: useEditableProps({
      id,
      editableProps
    })
  };
};

/**
 * {@link Editable} with plugins support.
 */

const EditablePlugins = props => {
  const {
    getEditableProps
  } = useSlatePlugins(props);
  return /*#__PURE__*/createElement(Editable, Object.assign({}, getEditableProps()));
};

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

function __rest(s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
}

const SlatePlugins = _a => {
  var {
    children
  } = _a,
      options = __rest(_a, ["children"]);

  const {
    getSlateProps,
    getEditableProps
  } = useSlatePlugins(options);
  const slateProps = getSlateProps();
  if (!slateProps.editor) return null;
  return /*#__PURE__*/React.createElement(Slate, Object.assign({}, slateProps), children, /*#__PURE__*/React.createElement(Editable, Object.assign({}, getEditableProps())));
};

/**
 * {@link useSlateStatic} with generic type.
 */

const useEditorStatic = () => useSlateStatic();
const useEditorSlate = () => useSlate();

/**
 * Editor id stored in `editor`.
 */

const useEditorId = () => useEditorStatic().id;

const getOptions = editor => {
  var _a, _b;

  return (_b = (_a = editor) === null || _a === void 0 ? void 0 : _a.options) !== null && _b !== void 0 ? _b : {};
};
const getPluginOptions = (editor, pluginKey) => {
  var _a;

  return (_a = getOptions(editor)[pluginKey]) !== null && _a !== void 0 ? _a : {};
};
const getPluginsOptions = (editor, pluginKeys) => pluginKeys.map(pluginKey => getPluginOptions(editor, pluginKey));
const getPluginType = (editor, pluginKey) => {
  var _a;

  return (_a = getPluginOptions(editor, pluginKey).type) !== null && _a !== void 0 ? _a : pluginKey;
};
const useEditorOptions = () => {
  return getOptions(useEditorStatic());
};
const useEditorPluginOptions = pluginKey => {
  return getPluginOptions(useEditorStatic(), pluginKey);
};
const useEditorPluginType = pluginKey => {
  return getPluginType(useEditorStatic(), pluginKey);
};

/*!
 * isobject <https://github.com/jonschlinkert/isobject>
 *
 * Copyright (c) 2014-2017, Jon Schlinkert.
 * Released under the MIT License.
 */

function isObject(val) {
  return val != null && typeof val === 'object' && Array.isArray(val) === false;
}

/*!
 * is-plain-object <https://github.com/jonschlinkert/is-plain-object>
 *
 * Copyright (c) 2014-2017, Jon Schlinkert.
 * Released under the MIT License.
 */

function isObjectObject(o) {
  return isObject(o) === true
    && Object.prototype.toString.call(o) === '[object Object]';
}

function isPlainObject(o) {
  var ctor,prot;

  if (isObjectObject(o) === false) return false;

  // If has modified constructor
  ctor = o.constructor;
  if (typeof ctor !== 'function') return false;

  // If has modified prototype
  prot = ctor.prototype;
  if (isObjectObject(prot) === false) return false;

  // If constructor does not have an Object-specific method
  if (prot.hasOwnProperty('isPrototypeOf') === false) {
    return false;
  }

  // Most likely a plain Object
  return true;
}

var History = {
  /**
   * Check if a value is a `History` object.
   */
  isHistory(value) {
    return isPlainObject(value) && Array.isArray(value.redos) && Array.isArray(value.undos) && (value.redos.length === 0 || Operation.isOperationList(value.redos[0])) && (value.undos.length === 0 || Operation.isOperationList(value.undos[0]));
  }

};
var SAVING = new WeakMap();
var MERGING = new WeakMap();
var HistoryEditor = {
  /**
   * Check if a value is a `HistoryEditor` object.
   */
  isHistoryEditor(value) {
    return History.isHistory(value.history) && Editor.isEditor(value);
  },

  /**
   * Get the merge flag's current value.
   */
  isMerging(editor) {
    return MERGING.get(editor);
  },

  /**
   * Get the saving flag's current value.
   */
  isSaving(editor) {
    return SAVING.get(editor);
  },

  /**
   * Redo to the previous saved state.
   */
  redo(editor) {
    editor.redo();
  },

  /**
   * Undo to the previous saved state.
   */
  undo(editor) {
    editor.undo();
  },

  /**
   * Apply a series of changes inside a synchronous `fn`, without merging any of
   * the new operations into previous save point in the history.
   */
  withoutMerging(editor, fn) {
    var prev = HistoryEditor.isMerging(editor);
    MERGING.set(editor, false);
    fn();
    MERGING.set(editor, prev);
  },

  /**
   * Apply a series of changes inside a synchronous `fn`, without saving any of
   * their operations into the history.
   */
  withoutSaving(editor, fn) {
    var prev = HistoryEditor.isSaving(editor);
    SAVING.set(editor, false);
    fn();
    SAVING.set(editor, prev);
  }

};

/**
 * The `withHistory` plugin keeps track of the operation history of a Slate
 * editor as operations are applied to it, using undo and redo stacks.
 */

var withHistory = editor => {
  var e = editor;
  var {
    apply
  } = e;
  e.history = {
    undos: [],
    redos: []
  };

  e.redo = () => {
    var {
      history
    } = e;
    var {
      redos
    } = history;

    if (redos.length > 0) {
      var batch = redos[redos.length - 1];
      HistoryEditor.withoutSaving(e, () => {
        Editor.withoutNormalizing(e, () => {
          for (var op of batch) {
            e.apply(op);
          }
        });
      });
      history.redos.pop();
      history.undos.push(batch);
    }
  };

  e.undo = () => {
    var {
      history
    } = e;
    var {
      undos
    } = history;

    if (undos.length > 0) {
      var batch = undos[undos.length - 1];
      HistoryEditor.withoutSaving(e, () => {
        Editor.withoutNormalizing(e, () => {
          var inverseOps = batch.map(Operation.inverse).reverse();

          for (var op of inverseOps) {
            // If the final operation is deselecting the editor, skip it. This is
            if (op === inverseOps[inverseOps.length - 1] && op.type === 'set_selection' && op.newProperties == null) {
              continue;
            } else {
              e.apply(op);
            }
          }
        });
      });
      history.redos.push(batch);
      history.undos.pop();
    }
  };

  e.apply = op => {
    var {
      operations,
      history
    } = e;
    var {
      undos
    } = history;
    var lastBatch = undos[undos.length - 1];
    var lastOp = lastBatch && lastBatch[lastBatch.length - 1];
    var overwrite = shouldOverwrite(op, lastOp);
    var save = HistoryEditor.isSaving(e);
    var merge = HistoryEditor.isMerging(e);

    if (save == null) {
      save = shouldSave(op);
    }

    if (save) {
      if (merge == null) {
        if (lastBatch == null) {
          merge = false;
        } else if (operations.length !== 0) {
          merge = true;
        } else {
          merge = shouldMerge(op, lastOp) || overwrite;
        }
      }

      if (lastBatch && merge) {
        if (overwrite) {
          lastBatch.pop();
        }

        lastBatch.push(op);
      } else {
        var batch = [op];
        undos.push(batch);
      }

      while (undos.length > 100) {
        undos.shift();
      }

      if (shouldClear(op)) {
        history.redos = [];
      }
    }

    apply(op);
  };

  return e;
};
/**
 * Check whether to merge an operation into the previous operation.
 */

var shouldMerge = (op, prev) => {
  if (op.type === 'set_selection') {
    return true;
  }

  if (prev && op.type === 'insert_text' && prev.type === 'insert_text' && op.offset === prev.offset + prev.text.length && Path.equals(op.path, prev.path)) {
    return true;
  }

  if (prev && op.type === 'remove_text' && prev.type === 'remove_text' && op.offset + op.text.length === prev.offset && Path.equals(op.path, prev.path)) {
    return true;
  }

  return false;
};
/**
 * Check whether an operation needs to be saved to the history.
 */


var shouldSave = (op, prev) => {
  if (op.type === 'set_selection' && op.newProperties == null) {
    return false;
  }

  return true;
};
/**
 * Check whether an operation should overwrite the previous one.
 */


var shouldOverwrite = (op, prev) => {
  if (prev && op.type === 'set_selection' && prev.type === 'set_selection') {
    return true;
  }

  return false;
};
/**
 * Check whether an operation should clear the redos stack.
 */


var shouldClear = op => {
  if (op.type === 'set_selection') {
    return false;
  }

  return true;
};

/**
 * {@link withHistory} that can be called multiple times without losing its history.
 */

const withHistoryPersist = editor => {
  if (HistoryEditor.isHistoryEditor(editor)) {
    const {
      history
    } = editor;
    const editorWithHistory = withHistory(editor);
    editorWithHistory.history = history;
    return editorWithHistory;
  }

  return withHistory(editor);
};
/**
 * @see {@link withHistoryPersist}
 */

const useHistoryPlugin = getSlatePluginWithOverrides(() => withHistory);

/**
 * @see {@link withReact}
 */

const useReactPlugin = getSlatePluginWithOverrides(() => withReact);

const isElement = Element.isElement;

const isAncestor = node => Editor.isEditor(node) || isElement(node);

const isText = Text.isText;

const isDescendant = node => isElement(node) || isText(node);

/**
 * Get inline types from the plugins
 */
const getInlineTypes = (editor, plugins) => {
  return plugins.flatMap(p => {
    var _a, _b;

    return (_b = (_a = p.inlineTypes) === null || _a === void 0 ? void 0 : _a.call(p, editor)) !== null && _b !== void 0 ? _b : [];
  });
};

/**
 * Get void types from the plugins
 */
const getVoidTypes = (editor, plugins) => {
  return plugins.flatMap(p => {
    var _a, _b;

    return (_b = (_a = p.voidTypes) === null || _a === void 0 ? void 0 : _a.call(p, editor)) !== null && _b !== void 0 ? _b : [];
  });
};

export { EditablePlugins, SlatePlugins, decoratePlugins, flatMapKey, getInitialState, getInlineTypes, getOptions, getPluginOptions, getPluginType, getPluginsOptions, getSetStateByKey, getSlatePluginWithOverrides, getStateById, getVoidTypes, isAncestor, isDescendant, isElement, isText, mergeState, onChangePlugins, onDOMBeforeInputPlugins, onKeyDownPlugins, pipe, renderElementPlugins, renderLeafPlugins, slatePluginsStore, useEditableProps, useEditorId, useEditorOptions, useEditorPluginOptions, useEditorPluginType, useEditorSlate, useEditorStatic, useHistoryPlugin, useInlineVoidPlugin, usePlugins, useReactPlugin, useSPEditor, useSlatePlugins, useSlatePluginsActions, useSlatePluginsEffects, useSlatePluginsOptions, useSlatePluginsPlugin, useSlatePluginsPluginKeys, useSlatePluginsStore, useSlatePluginsValue, useSlateProps, withHistoryPersist, withInlineVoid, withSlatePlugins };
//# sourceMappingURL=index.es.js.map
